# Introduction

The Entity Admin Handlers module provides an out-of-the box admin UI for custom
entity types whose configuration means that the admin UI code in Drupal core
doesn't fit.

This reduces the need for boilerplate code to provide a UI to manage fields on
the entity type and its bundles.

This module provides custom entity type handlers, which must be added to the
annotation of an entity type (or attribute, on newer versions of Drupal core).

The entity type handlers are grouped into different sets in this module's /src
folder, for different entity type configurations.

Each set provides provides:

- a route provider handler
- an entity links handler
- a controller for the routes that are defined by the route provider handler

The RouteProvider class's documentation has details specific to the set.

## Requirements

This module expects the patch at
https://www.drupal.org/project/drupal/issues/2976861 to be applied in order to
provide menu, task, and action links. The routes will work without it, however.

## Usage

### Route provider

Set the 'route_provider' to the handler from the set you require.

For example:

```
 *     "route_provider" = {
 *       "html" = "Drupal\entity_admin_handlers\SingleBundleEntity\SingleBundleEntityHtmlRouteProvider",
 *     },
```

If you have generated the entity type using Module builder, you should replace
the 'html' route provider it added.

### Link provider

If you are using the core patch, set the 'link_provider' to the handler from the set you require.

For example:

```
 *     "link_provider" = "Drupal\entity_admin_handlers\SingleBundleEntity\SingleBundleEntityLinksProvider",
```

### Links plugin files

If you are not using the core patch, you will need to add links YAML files for
menu and task plugins.

Copy and rename the files in the 'links_templates' folder in the set, and
replace the ID of your entity type where marked.

### Other properties

The entity type must also define:

- a 'field-ui-base' link template in the 'links' section of the entity type
  annotaion. This will typically be of the form
  '/admin/structure/MY_ENTITY_TYPE'.
- an admin permission in the 'admin_permission' property
- the field_ui_base_route property, which must be set to
  'entity.ENTITY_TYPE.field_ui_base'.

## Entity type configurations

The following cases are provided for.

## Single bundle entity type

This is for entity types that do not use bundles and in effect, have a single
bundle which typically has the same name as the entity type.

This is identical to the way the core user entity type works.

The handlers define a dummy route for Field UI module to hang its routes off.

## Plain bundle entity type

This is for entity types that use multiple bundles, but do not have a config
entity type that defines the bundles (so for example, if the content entity type
is 'foo', there is no 'foo_type' entity type).

Such an entity type may have its bundles defined in hook_entity_bundle_info()
implementations, or have them derived from plugins using the Entity API contrib
module's bundle plugins functionality.

The handlers define a route that lists the bundles, and a dummy route for each
bundle for Field UI module to hang its routes off.
